#!/bin/sh

# Usage:
# prepare_update.sh factory_system factory_mount_point 
#                   current_system_mount_point temp_mount_point
#                   update_server update_filename method
# 
# factory_system = partition of factory default system
# factory_mount_point = mount point for factory default system
# current_system_mount_point = where to move current root to
# temp_mount_point = mount point for tmpfs filesystem
# update_server = IP address of update server
# update_filename = name of the update file on the server
# method = automatic if run from automatic check otherwise manual or empty
#
# Prepares the filesystem for an update by switching root to
# factory system and mounting old root to given mountpoint


# Check parameters and define default values if needed 
# Some paths are without leading / because of pivot_root
if [ -z $1 ] ; then factory_system=/dev/mtdblock/3
else factory_system=$1
fi
if [ -z $2 ] ; then factory_mount_point=factory_system
else factory_mount_point=$2
fi
if [ -z $3 ] ; then current_system_mount_point=current_system
else current_system_mount_point=$3
fi
if [ -z $4 ] ; then temp_mount_point=/temporary_files
else temp_mount_point=$4
fi
if [ -z $5 ] ; then update_server=190.16.7.244
else update_server=$5
fi
if [ -z $6 ] ; then update_filename=tester.tar.gz
else update_filename=$6
fi
if [ -z $7 ] ; then method=manual
else method=$7
fi

# Check for moint point and create if necessary
if [ ! -d /$factory_mount_point ]; then 
   mkdir /$factory_mount_point
fi

# Mount factory system
mount $factory_system /$factory_mount_point -t jffs2

# Check for other moint points and create if necessary
if [ ! -d /$current_system_mount_point ]; then 
   mkdir /$current_system_mount_point
fi
if [ ! -d /$factory_mount_point/$current_system_mount_point ]; then 
   mkdir /$factory_mount_point/$current_system_mount_point
fi
if [ ! -d $temp_mount_point ]; then 
   mkdir $temp_mount_point
fi
if [ ! -d /$factory_mount_point$temp_mount_point ]; then 
   mkdir /$factory_mount_point$temp_mount_point
fi

if [ $method = "manual" ] ; then
   # Mount tmpfs
   mount -t tmpfs /dev/shm $temp_mount_point

   # Download update file
   cd $temp_mount_point
   tftp -g -r$update_filename -l$update_filename $update_server
fi

# Check wether update file is avaiable
if [ -s $temp_mount_point/$update_filename ] ; then

   # Change root filesystem to factory system 
   cd /$factory_mount_point
   pivot_root . $current_system_mount_point 

   # Update system
   chroot . sh /sbin/update_system.sh $update_filename /$current_system_mount_point $temp_mount_point 

   # Change root filesystem back to updated system
   cd /$current_system_mount_point
   pivot_root . $factory_mount_point
   
   # Mark current system as updated
   fw_setenv bootflag updated
   fw_setenv bootcmd "setenv bootcmd run failsave ; saveenv ; run flash_all"

else
   # There was a problem with the download
   echo Update $update_filename could not be loaded from server $update_server
   cd /
fi

# Unmount file systems
umount /$factory_mount_point
if [ $method = "manual" ] ; then
umount $temp_mount_point
fi

