#!/bin/sh

# Usage:
# update_check.sh
# 
# Checks wether we had an update and:
# - the system is running fine
# - the system crashed and we are now running
#   the factory system (switch to maintenance mode)
# - the system is now in maintenance mode


# Function to easyly access environment variables
get_env_value()
{
   echo `grep ^$1 $tempfile | cut -d= -f2`
}

# Store U-Boot enviromment in a temporary file
tempfile=/tmp/u-boot.environment
fw_printenv > $tempfile


# Get boot flag
bootflag=`get_env_value "bootflag"`
# Did we just update?
if [ ${bootflag:="running"} = "updated" ]; then
   # Now check booted system
   # factory_system means the updated filesystem did not 
   # boot properly
   # failsave means we are running the new filesystem i.e. OK
   booted_system=`get_env_value "bootcmd"| cut -d " " -f2`
   if [ ${booted_system:="custom"} = "factory_system" ] ; then
      fw_setenv bootflag maintenance
      echo "==================================="
      echo "   Installed update not working!"
      echo " System is now in maintanence mode"
      echo "Please contact system administrator"
      echo "==================================="
      exit 1
   else
      if [ $booted_system = "failsave" ] ; then
         echo "====================================="
         echo "Update succesfull. Confirming changes"
         echo "====================================="
         fw_setenv bootflag running
         fw_setenv bootcmd run flash_all
	 exit 0
      else
         echo "====================================="
         echo " Unknown system booted after update"
         echo "      Assuming everything is OK"
         echo "Boot factory_system if problems occur"
         echo "====================================="
         fw_setenv bootflag running
         fw_setenv bootcmd run flash_all
	 exit 0
      fi
      
   fi
fi

# Check if automatic update enabled
update_method=`get_env_value "update_method"`
if [ ${update_method:="manual"} = "automatic" ]; then

   echo Checking for update
   update_server=`get_env_value "update_server"`
   echo Using server ${update_server:="192.168.1.1"}
   current_revision=`cat /etc/revision`
   echo -n "Current revision is ${current_revision:=1}. "
   let current_revision=$current_revision+1
   echo Trying revision $current_revision
   
   factory_system=/dev/mtdblock/3
   factory_mount_point=factory_system
   current_system_mount_point=current_system
   temp_mount_point=/temporary_files
   update_filename=incaip-update-rev$current_revision.tar.gz
   temp_update_storage=/update_storage
   
   echo Trying file $update_filename
 
   if [ ! -d $temp_update_storage ]; then 
      mkdir $temp_update_storage
   fi
   
   # Mount tmpfs
   mount -t tmpfs /dev/shm $temp_update_storage

   # Download update file
   cd $temp_update_storage
   tftp -g -r$update_filename -l$update_filename $update_server

   # Check wether update file is avaiable
   if [ -s $temp_update_storage/$update_filename ] ; then
      echo Revision $current_revision avaiable. Starting update...
      /sbin/prepare_update.sh /dev/mtdblock/3 factory_system current_system $temp_update_storage $update_server $update_filename automatic
      reboot
   else 
      echo No new update avaiable
   fi   
   cd /
   umount $temp_update_storage
fi

# Remove temporary file
rm $tempfile
